import re


class DateExtractor:
    """
    Extracts English or Bangla date from string.

    Args:
        sentence (str): The input string.

    Returns:
        list: A list containing all the dates found in the sentence.
    """

    @staticmethod
    def get_regex_patterns():
        en_month_name = [
            "jan",
            "feb",
            "mar",
            "apr",
            "may",
            "june",
            "july",
            "aug",
            "sept",
            "oct",
            "nov",
            "dec",
        ]
        bn_month_name = [
            "জানুয়ারি",
            "ফেব্রুয়ারি",
            "মার্চ",
            "এপ্রিল",
            "মে",
            "জুন",
            "জুলাই",
            "অগাস্ট",
            "আগস্ট",
            "সেপ্টেম্বর",
            "অক্টোবর",
            "নভেম্বর",
            "ডিসেম্বর",
        ]

        en_dd_mm_yy = "\\d{1,4}(( )+)?[-\\/,.;: ](( )+)?\\d{1,4}(( )+)?[-\\/,.;: ](( )+)?\\d{1,4}\\b"
        en_number_month_year = (
            "\\d{1,2}[-\\/,.;: ](( )+)?("
            + "|".join(en_month_name)
            + ")[a-z]{0,6}((( )+)?[-\\/,.;: ](( )+)?\\d{1,4})?\\b"
        )
        en_month_number_year = (
            "("
            + "|".join(en_month_name)
            + ")[a-z]{0,6}(( )+)?[-\\/,.;: ](( )+)?\\d{1,2}([-\\/,.;: ](( )+)?\\d{1,4})?\\b"
        )
        en_month_year = (
            "("
            + "|".join(en_month_name)
            + ")[a-z]{0,6}(( )+)?[-\\/,.;: ](( )+)?\\d{1,4}\\b"
        )
        # en_plain_date ='\\d{8}\\b'

        bn_dd_mm_yy = "[০-৯]{1,4}(( )+)?[-\\/,.;: ](( )+)?[০-৯]{1,4}(( )+)?[-\\/,.;: ](( )+)?[০-৯]{1,4}"
        bn_number_month_year = (
            "[০-৯]{1,2}[-\\/,.;: ](( )+)?("
            + "|".join(bn_month_name)
            + ")((( )+)?[-\\/,.;: ](( )+)?[০-৯]{1,4})?"
        )
        bn_month_number_year = (
            "("
            + "|".join(bn_month_name)
            + ")(( )+)?[-\\/,.;: ][০-৯]{1,2}(?![০-৯])((( )+)?[-\\/,.;: ](( )+)?[০-৯]{1,4})?"
        )
        bn_month_year = (
            "(" + "|".join(bn_month_name) + ")(( )+)?[-\\/,.;: ](( )+)?[০-৯]{1,4}"
        )
        bn_year_date_num_month_name = (
            "[০-৯]{1,4}(( )+)?[-\\/,.;: ](( )+)?(সালে|সালের)(( )+)?[-\\/,.;: ]?[০-৯]{1,2}(( )+)?[-\\/,.;: ]?(( )+)?("
            + "|".join(bn_month_name)
            + ")?"
        )
        # bn_plain_date = "[০-৯]{8}"

        en_regex = "|".join(
            [en_dd_mm_yy, en_number_month_year, en_month_number_year, en_month_year]
        )  # , en_plain_date])
        bn_regex = "|".join(
            [
                bn_dd_mm_yy,
                bn_number_month_year,
                bn_month_number_year,
                bn_month_year,
                bn_year_date_num_month_name,
            ]
        )  # , bn_plain_date])
        combined_regex = en_regex + "|" + bn_regex
        return combined_regex

    def __init__(self) -> None:
        self.date_regex = self.get_regex_patterns()

    def get_dates(self, sentence):
        dates = re.finditer(self.date_regex, sentence, re.IGNORECASE)
        return [date.group() for date in dates]


if __name__ == "__main__":

    English_sentences = [
        "I'm planning a beach vacation in July 2023.",
        "The conference is scheduled for September 05, 2023.",
        "We'll have a family gathering on December 25, 2023, for Christmas.",
        "My sister's wedding is on March 18, 2023.",
        "I'm looking forward to my birthday on August 12, 2023.",
        "Let's go skiing in February 2023.",
        "Our company picnic is set for June 30, 2023.",
        "I have an important exam on May 10, 2023.",
        "The new semester starts in January 2023.",
        "We're planning a road trip in April 2023.",
        "I'll be moving to a new apartment in October 2023.",
        "Mark your calendar for November 05, 2023, it's Diwali.",
        "I'm excited for the summer camp in July 2023.",
        "The concert tickets for March 25, 2023, are selling fast.",
        "Let's have a barbecue party in August 2023.",
        "I'll be visiting my grandparents in May 2023.",
        "I'm attending a wedding in September 2023.",
        "We're planning a trip to Europe in June 2023.",
        "I have a doctor's appointment on April 10, 2023.",
        "The company's annual meeting is in February 2023.",
        "We're celebrating Valentine's Day on February 14, 2023.",
        "My graduation ceremony is on June 15, 2023.",
        "I'm starting a new job in March 2023.",
        "I'm planning to go hiking in the mountains in May 2023.",
        "The school play is scheduled for November 2023.",
        "Let's go pumpkin picking in October 2023.",
        "I have a dentist appointment in January 2023.",
        "The summer festival is in July 2023.",
        "We're hosting a Thanksgiving dinner in November 2023.",
        "I'm going to a music festival in April 2023.",
    ]
    template_provided = [
        "০১-এপ্রিল/২০২৩",
        "১ এপ্রিল ২০২৩",
        "2023-04-05 df",
        "06-04-2023 df",
        "04/01/2023 er",
        "07 April, 2023 er",
        "Apr 1, 2023 er",
        "2023/04/01 er",
        "01-Apr-2023 erv",
        "01-Apr/2023 sere",
        # "20230401 ",
        # "20042024 ",
    ]
    Bangla_sentences = [
        "১৯৯৬ সালের ৬তারিখে নির্ধারিত করা হয়েছে.",
        "১৯৯৬ সালের৬ সেপ্টেম্বর ভ্রমণ পরিকল্পনা করছি.",
        "আমি জুলাই ২০২৩ তে একটি সমুদ্র ভ্রমণ পরিকল্পনা করছি.",
        "সম্মেলনটি সেপ্টেম্বর ০৫ ২০২৩ তারিখে নির্ধারিত করা হয়েছে.",
        "আমরা খ্রীষ্টমাসের জন্য ডিসেম্বর ২৫ ২০২৩ তারিখে পরিবারের সংগঠন করব.",
        "আমার বোনের বিয়ে ১৮ মার্চ, ২০২৩ তারিখে.",
        "আমি আগামী ১২ আগস্ট, ২০২৩ তারিখে আমার জন্মদিনে দেখা করছি.",
        "আমরা ফেব্রুয়ারি ২০২৩ তে স্কিউইং যাব.",
        "আমাদের কোম্পানির পিকনিকটি জুন ৩০ ২০২৩ তারিখে নির্ধারিত হয়েছে.",
        "আমার গুরুত্বপূর্ণ পরীক্ষা ১০ মে, ২০২৩ তারিখে.",
        "নতুন সেমিস্টার শুরু হয় জানুয়ারি ২০২৩ তারিখে.",
        "আমরা এপ্রিল ২০২৩ তে একটি রোড ট্রিপ পরিকল্পনা করছি.",
        "আমি অক্টোবর ২০২৩ তারিখে একটি নতুন অ্যাপার্টমেন্টে যাচ্ছি.",
        "আপনার ক্যালেন্ডারের মার্ক নভেম্বর ০৫, ২০২৩, এটি দীপাবলি.",
        "আমি জুলাই ২০২৩ তারিখে গরম শিবিরের জন্য উৎসাহিত.",
        "মার্চ ২৫, ২০২৩ তারিখের কনসার্ট টিকেটগুলি দ্রুত বিক্রয় হচ্ছে.",
        "আগস্ট ২০২৩ তারিখে একটি বারবিকিউ পার্টি আয়োজন করা হবে.",
        "আমি মে ২০২৩ তারিখে আমার নানা-নানির কাছে যাচ্ছি.",
        "আমি সেপ্টেম্বর ২০২৩ তারিখে একটি বিয়ে অনুষ্ঠানে যাচ্ছি.",
        "আমরা জুন ২০২৩ তে ইউরোপে একটি ভ্রমণ পরিকল্পনা করছি.",
        "আমার ডেন্টিস্টের অ্যাপয়েন্টমেন্ট এপ্রিল ১০, ২০২৩ তারিখে.",
        "কোম্পানির বার্ষিক সভা ফেব্রুয়ারি ২০২৩ তারিখে হয়.",
        "আমার অনুষ্ঠান সমাপন হয়েছে জুন ১৫, ২০২৩ তারিখে.",
        "আমি মার্চ ২০২৩ তারিখে একটি নতুন চাকরি শুরু করছি.",
        "আমি মে ২০২৩ তারিখে পাহাড়ে ট্রেকিং করতে যাচ্ছি.",
        "বিদ্যালয়ের নাটকটি নভেম্বর ২০২৩ তারিখে নির্ধারিত হয়েছে.",
        "আমরা অক্টোবর ২০২৩ তার",
        "১৯৯৬ সালের ৬ সেপ্টেম্বর",
    ]

    combined_sentences = English_sentences + template_provided + Bangla_sentences
    date_extractor = DateExtractor()

    for sentence in combined_sentences:
        dates = date_extractor.get_dates(sentence)
        print(f"Input: {sentence}: Output: {dates}")

    # print(f"Input: {sentence}: Output: {[date.group() for date in dates]}")



import re
from .parser import NumberParser, TextParser
from .config import Config as cfg

npr, tp = NumberParser(), TextParser()


class Word2NumberMap:

    def __init__(self):
        pass

    def equation_of_sare_and_der(self, value: str, fraction: float) -> float:
        """
        Convert word if start bangla word like "সাড়ে", সারে, দেড়, দের
        and return numerical value
        """
        re_value = int(value.replace(value[0], "1"))
        return (re_value * fraction) + int(value)

    def equation_of_arai(self, value: str, fraction: float) -> float:
        """
        Convert word if the start bangla word like "আড়াই" or "আরাই"
        and return numerical value
        """
        return int(value) * fraction

    # def call_function()

    def adjust_value_conversion(self, value, sum_status=False):
        """
        Convert adjust value with numerical representation

        """

        status, adjust_name = False, ""
        for v in value:
            if v in cfg.adjust_number:
                adjust_name = v
                status, sum_status = True, False
                break
        if status:
            if len(value) == 3 and value[1].isdigit() and value[2].isdigit():
                number = str(int(value[1]) + int(value[2]))
            else:
                number = str(value[1])

            fraction_value = cfg.adjust_number[adjust_name]
            function_name = cfg.function_mapping[adjust_name]

            if hasattr(self, function_name) and callable(getattr(self, function_name)):
                func = getattr(self, function_name)
                return_value = func(number, fraction_value)
                return str(int(return_value)), sum_status

        return value, sum_status

    def check_last_chars(self, word: str) -> bool:
        """
        Checking last character match with target character
        """
        for char in cfg.target_chars:
            if word.endswith(char):
                return True, char
        return False, None

    def sum_status(self, lst: list) -> bool:
        """
        Cheching the group word sum status

        """
        status_list = []
        for sublist in lst:
            x = [
                "1" if i in sublist else "0"
                for i in cfg.checking_hunderds + cfg.checking_adjust
            ]
            if "1" in x:
                status_list.append(True)
            else:
                status_list.append(False)
        return status_list

    def word_clustering(self, input_list: list) -> list:
        """
        Grouping of the word from the list of text
        """
        output, temp_sequence = [], []
        i = 0
        while i < len(input_list):
            input_list[i] = input_list[i].replace("শত00", "শত")
            input_list[i] = input_list[i].replace("শো00", "শত")
            if input_list[i].isdigit():
                if len(input_list[i]) == 2 and len(input_list) - 1 != i:
                    if input_list[i + 1] in cfg.checking_hunderds:
                        temp_sequence.append(input_list[i])
                        output.append(temp_sequence)
                    else:
                        temp_sequence.append(input_list[i])
                        output.append(temp_sequence)
                        temp_sequence = []

                elif len(input_list) - 1 == i:
                    temp_sequence.append(input_list[i])
                    output.append(temp_sequence)
                else:
                    temp_sequence.append(input_list[i])
            elif (
                input_list[i] in cfg.decimale_chunks
                or input_list[i] in cfg.fraction_int
            ):
                temp_sequence.append(input_list[i])
            elif input_list[i] in cfg.hundreds:
                temp_sequence.append(input_list[i])
            elif input_list[i] in cfg.checking_conjugative_number:
                temp_sequence.append(input_list[i])
            elif input_list[i] in cfg.en_doshok_map:
                temp_sequence.append(input_list[i])
            elif input_list[i] in cfg.adjust_number:
                temp_sequence.append(input_list[i])
            else:
                if temp_sequence:
                    output.append(temp_sequence)
                    temp_sequence = []
            i += 1
        return output

    def checking_hundreds_only(self, input_list: list) -> bool:
        """
        Checking status if all are handerds word

        """
        all_numeric_status = all(item.isdigit() for item in input_list)
        if all_numeric_status:
            return False
        for item in input_list:
            if (
                item in cfg.decimale_chunks
                or item in cfg.adjust_number
                or item in cfg.fraction_int
                or item in cfg.conjugative_number
                or item in cfg.en_doshok_map
            ):
                return False
        return True

    def clustring_consecutive_hunderd(self, input_list: list) -> [list, list]:
        """
        Clustering consecutinve handerd with sum status

        """
        temp, output_list, output_status = [], [], []
        for i, value in enumerate(input_list):
            if value in cfg.bn_hundreds:
                temp.append(value)
                if i == len(input_list) - 1 or not input_list[i + 1].isdigit():
                    output_list.append(temp), output_status.append(False)
                    temp = []
            elif value.isdigit():
                temp.append(value)
                if i == len(input_list) - 1 or not input_list[i + 1].isdigit():
                    output_list.append(temp), output_status.append(True)
                    temp = []
            else:
                output_list.append([value]), output_status.append(False)
        return output_list, output_status

    # def find_word_index(self, text:str, word:str)->list:
    #     """
    #     Word spanning position
    #     """
    #     start  = text.find(word)
    #     end = start+len(word)
    #     return [start, end]

    # def replace_text_at_position(self, text:str, replacement:str, start_pos:int, end_pos:int)->str:
    #     """
    #     Replance text using text position

    #     """
    #     return text[:start_pos] + replacement + text[end_pos:]

    def converting_condition(
        self, word: str, final_value: list, c_data: list, index: int
    ) -> [list, int]:
        """
        Convert word to conditional mapping with digits
        """

        if word.isdigit():
            final_value.append(word)
        elif word in cfg.decimale_chunks:
            if final_value:
                value = final_value[-1]
                d_c = int(cfg.decimale_chunks[word])
                if value.isdigit():
                    final_value.append(str((d_c * int(value)) - int(value)))
                else:
                    final_value.append(d_c)
            else:
                final_value.append(int(cfg.decimale_chunks[word]))
        elif word in cfg.bn_hundreds:
            final_value.append(cfg.bn_hundreds[word])
        elif word in cfg.en_doshok_map:
            final_value.append(cfg.en_doshok_map[word])
        elif word in cfg.fraction_int:
            final_value.append(cfg.fraction_int[word])
        elif word in cfg.conjugative_number:
            c_n = int(cfg.conjugative_number[word]) - 1
            if len(c_data) > index + 1:
                l_value = [str(c_data[index + 1])] * c_n
            else:
                l_value = cfg.conjugative_number[word]
            final_value.extend(l_value)
        else:
            final_value.append(word)
        return final_value, index

    def converting2digits(
        self, results: list, text_list: list, sum_status_list: list
    ) -> str:
        """

        Converting word to digit and if have hunderds only then cluster again

        """
        original_text = " ".join(text_list)
        for result_chunk, status in zip(results, sum_status_list):

            # checking hunderds only and return status
            hundreds_status = self.checking_hundreds_only(result_chunk)

            # generate number clustring
            if hundreds_status:
                clustring_data, clustring_status = self.clustring_consecutive_hunderd(
                    result_chunk
                )
            else:
                clustring_data, clustring_status = [result_chunk], [status]
            for c_data, c_status in zip(clustring_data, clustring_status):
                replance_text = " ".join(c_data)
                word_spanning = npr.find_word_index(original_text, replance_text)
                index, final_value = 0, []
                for c_d in c_data:
                    final_value, index = self.converting_condition(
                        c_d, final_value, c_data, index
                    )
                    index += 1
                value, status = self.adjust_value_conversion(
                    final_value, sum_status=c_status
                )
                if status:
                    numbers = str(sum(int(num) for num in value))
                elif isinstance(value, str):
                    numbers = value
                else:
                    numbers = "".join(value)

                original_text = npr.replace_text_at_position(
                    original_text, numbers, word_spanning[0], word_spanning[1]
                )
        return original_text

    def replace_word_to_number(self, text: list) -> list:
        """
        Word to numerical digit conversation

        """
        index = 0
        for t in text:
            if t in cfg.bn_word_map:
                text[index] = cfg.bn_word_map[t]
            elif t in cfg.en_number_mapping:
                text[index] = cfg.en_number_mapping[t]
            else:
                pass
            index += 1
        return text

    def normalize(self, text: str) -> list:
        """

        This funcation normalize the text like white space and decimal number

        Arg:
            text{string}    : input string

        Return:
            text_list{list} : process text list space spliting

        """
        text = tp.collapse_whitespace(text)
        texts = text.split(" ")
        text_list = []
        for word in texts:
            status, char = self.check_last_chars(word)
            if status:
                if word not in cfg.bn_word_map:
                    if word[:-2] in cfg.bn_word_map:
                        rword = npr.number_processing(
                            str(int(cfg.bn_word_map[word[:-2]]) * 100)
                        )
                        rword = rword.replace(" শত", "শত")
                        text_list.extend(rword.split(" "))
                    else:
                        text_list.append(word)
                else:
                    text_list.append(word)
            else:
                text_list.append(word)
        return text_list

    def convert_word2number(self, text):
        text = self.normalize(text + " ")
        text_list = self.replace_word_to_number(text)
        results = self.word_clustering(text_list)
        sum_status_list = self.sum_status(results)
        text = self.converting2digits(results, text_list, sum_status_list)
        return text


if __name__ == "__main__":

    texts = [
        "আমি এক দুই তিন চার পাঁচ টু থ্রি ফাইভ ছয় সেভেন এইট নাইন শূন্য আমার ফোন নাম্বার জিরো ওয়ান ডাবল সেভেন",
        "ওয়ান ডাবল নাইন টু",
        "একশ বিশ টাকা",
        "জিরো টু ডাবল ওয়ান",
        "জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন",
        "আমার ফোন নম্বর জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন",
        "ট্রিপল টু ওয়ান",
        "দুই হাজার চারশো বিশ",
        "দুই হাজার চারশ  বিশ",
        "হাজার বিশ",
        "ডাবল নাইন টু",
        "এক লক্ষ চার হাজার দুইশ",
        "এক লক্ষ চার হাজার দুইশ এক",
        "এক লক্ষ চার হাজার দুইশ এক টাকা এক দুই",
        "আমাকে এক লক্ষ দুই হাজার টাকা দেয়",
        "আমাকে এক লক্ষ দুই হাজার এক টাকা দেয় এন্ড তুমি বিশ হাজার টাকা নিও এন্ড এক লক্ষ চার হাজার দুইশ এক টাকা এক ডবল দুই",
        "ছয় হাজার বিশ",
        "আমার সাড়ে পাঁচ হাজার",
        "আমার সাড়ে তিনশ",
        "আড়াই হাজার",
        "আড়াই লক্ষ",
        "ডেরশ",
        "আমাকে ডেরশ টাকা দেয়",
        "সাড়ে পাঁচ কোটি টাকা",
        "সাড়ে 1254 টাকা",
        "জিরো",
        "একশ বিশ take একশ",
        "জিরো টু ডাবল ওয়ান",
        "জিরো টু ওয়ান ওয়ান",
        "থ্রি ফোর ফাইভ এইট",
        "একশ বিশ টাকা",
        "ডাবল ওয়ান ডবল টু",
        "জিরো ওয়ান টু",
        "থ্রি ফোর ফাইভ সিক্স",
        "সেভেন এইট নাইন টেন",
        "একশ দুইশ তিনশ",
        "চারশ পাঁচশ",
        "ছয়শ সাতশ",
        "আটশ নয়শ",
        "দশ তিরানব্বই",
        "ট্রিপল থ্রি টু",
        "শূন্য এক দুই তিন",
        "চার পাঁচ ছয় সাত",
        "আট নয় দশ এগারো",
        "বারো তেরো চৌদ্দ পনেরো",
        "ষোল সতেরো আঠারো উনিশ",
        "বিশ একুশ বাইশ তেইশ",
        "চব্বিশ পঁচিশ ছাব্বিশ সাতাশ",
        "আঠাশ ঊনত্রিশ ত্রিশ একত্রিশ",
        "বত্রিশ তেত্রিশ চৌত্রিশ পঁয়ত্রিশ",
        "ছত্রিশ সাঁইত্রিশ আটত্রিশ ঊনচল্লিশ",
        "চল্লিশ একচল্লিশ বিয়াল্লিশ তেতাল্লিশ",
        "চুয়াল্লিশ পঁয়তাল্লিশ ছেচল্লিশ সাতচল্লিশ",
        "আটচল্লিশ ঊনপঞ্চাশ পঞ্চাশ একান্ন",
        "বাহান্ন তিপ্পান্ন চুয়ান্ন পঞ্চান্ন",
        "ছাপ্পান্ন সাতান্ন আটান্ন ঊনষাট",
        "ষাট একষট্টি বাষট্টি তেষট্টি",
        "চৌষট্টি পঁয়ষট্টি ছেষট্টি সাতষট্টি",
        "আটষট্টি ঊনসত্তর সত্তর একাত্তর",
        "বাহাত্তর তিয়াত্তর চুয়াত্তর পঁচাত্তর",
        "ছিয়াত্তর সাতাত্তর আটাত্তর ঊনআশি",
        "আশি একাশি বিরাশি তিরাশি",
        "চুরাশি পঁচাশি ছিয়াশি সাতাশি",
        "আটাশি ঊননব্বই নব্বই একানব্বই",
        "বিরানব্বই তিরানব্বই চুরানব্বই পঁচানব্বই",
        "ছিয়ানব্বই সাতানব্বই আটানব্বই নিরানব্বই",
        "এক লক্ষ চার হাজার দুইশ এক টাকা এক দুই",
        "তিনশ পঁচিশ পাঁচশ",
        "তিনশ পঁচিশ পাঁচশ এক",
        "চা-পুন",
        "ওকে",
        "ডের আউটস্ট্যান্ডিং কত",
        "ডাবল",
        "নাইন ডাবল এইট",
        "দশ বারো এ এগুলা একশ একশ দুই",
    ]

    # texts = ["দশ বারো এ এগুলা একশ একশ দুই"]

    wnm = Word2NumberMap()
    for i in texts:
        print("=" * 40)
        print("input : ", i)
        text = wnm.convert_word2number(i)
        print("output : ", text)
        print("=" * 40)



import sys
from pathlib import Path
sys.path[0] = str(Path(sys.path[0]).parent)

import time
from pybangla.module.main import Normalizer

if __name__ == "__main__":


    # Testing Date format
    date_list = [
        "০১-এপ্রিল/২০২৩",
        "১ এপ্রিল ২০২৩" 
        "2023-04-05",  
        "06-04-2023", 
        "04/01/2023",  
        "07 April, 2023", 
        "Apr 1, 2023",  
        "2023/04/01", 
        "01-Apr-2023", 
        "01-Apr/2023",  
        "20230401",  
        "20042024",
        ["1", "4", "2025"]
    ]
    # number = "123456" or "২০২৩"
    number = "২০২৩"
    nmlr = Normalizer()
    print("++++++++++++++++++++ Date testing ++++++++++++++++++++++")
    print("Date format Testing : ", end ="", flush=True)
    for date_ in date_list:
        start_time = time.time()
        formated_date = nmlr.date_format(date_, language="bn")
        print(formated_date)
    print("++++++++++++++++++++ end of Date testing ++++++++++++++++++++++")

    print("++++++++++++++++++++ en number to bn number convert ++++++++++++++++++++++")
    number = nmlr.number_convert(number, language="bn")

    print("Bn Number : ", number)
    print("++++++++++++++++++++ stop number convert ++++++++++++++++++++++")

    print("++++++++++++++++ Today +++++++++++++++++++++")
    today_date = nmlr.today(language="bn")
    # today_date = nmlr.today(language="en")
    print(today_date)

    print("++++++++++++++++ End Today +++++++++++++++++++++")
    # print(today_date)
    print("++++++++++++++++ weekdays +++++++++++++++++++++")
    # weekdays = nmlr.weekdays()
    # weekdays = nmlr.weekdays(language="bn")
    # weekdays = nmlr.weekdays(language="en")
    # weekdays = nmlr.weekdays(day = "সোমবার")
    weekdays = nmlr.weekdays(day = "Monday")
    print(weekdays)

    print("++++++++++++++++ end weekdays +++++++++++++++++++++")
    # print(weekdays)
    print("+++++++++++++++ seasons ++++++++++++++++++++++++")
    # seasons = nmlr.seasons()
    # seasons = nmlr.seasons(language="bn")
    # seasons = nmlr.seasons(language="en")
    seasons = nmlr.seasons(seasons = "গ্রীষ্ম")
    print(seasons)

    print("+++++++++++++++ end seasons ++++++++++++++++++++++++")

    print("+++++++++++++++++ months +++++++++++++++++++++++++++")
    month = nmlr.months()
    month = nmlr.months(month="মার্চ")
    print(month)


    text = "রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম, ১২৩৪ শতাব্দীতে ¥২০৩০.১২৩৪ বিবিধ  বাকেরগঞ্জ উপজেলার প্রায় 40 ভাগের পেশাই চাষাবাদ 80 and 40 ২২"
    
    text = nmlr.text_normalizer(text)

    print(text)
    print("+++++++++++++++++ end months +++++++++++++++++++++++++++")
    
    print("+++++++++++++++++ punctuations +++++++++++++++++++++++++++")
    text = "শাশ্বতী জানালেন, বিয়ে-বৌভাতের চলতি মরশুমে ভোজবাড়ির ডিনারে এক জনের প্রত্যাবর্তন ঘটেছে— দাপুটে প্রত্যাবর্তন।"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "সে যা-ই হোক, সত্যিকারের এমন পাকা পোনা শেষ বার নেমন্তন্ন বাড়িতে খেয়েছি ১৯৮৭-র এপ্রিলে।"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "‘আমি ছেলেকে ছাড়া বাঁচব কী করে’"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "২০২০ সালে ‘দেয়ার ইজ নো এভিল’ সিনেমার জন্য বার্লিন উৎসবের সর্বোচ্চ পুরস্কার স্বর্ণভালুক জেতেন রাসুলফ। ২০২২ সালে তাঁকে গ্রেপ্তার করা হয়, পরে ইরানে বিক্ষোভ শুরু হলে সে বছরের সেপ্টেম্বর মাসে তাঁকে মুক্তি দেওয়া হয়।"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "করে ‘দ্য পাবলিক গ্যাম্বলিং অ্যাক্ট ১৮৬৭’ সংশোধনের উদ্যোগ নেওয়া হয়েছে।"
    text = nmlr.text_normalizer(text)
    print(text) 
    
    text = "৬৭% সংশোধনের উদ্যোগ নেওয়া হয়েছে।"
    text = nmlr.text_normalizer(text)
    print(text)
    print("+++++++++++++++++ end punctuations +++++++++++++++++++++++++++")
    
    print("+++++++++++++++++ random test +++++++++++++++++++++++++++")
    text = '"আবূ মুসা আশা’আরী (রাঃ) থেকে বর্ণিতঃরাসূলুল্লাহ (সাল্লাল্লাহু ‘আলাইহি ওয়া সাল্লাম) বলেছেন, “তিনবার অনুমতি চাইবে। অনুমতি দিলে প্রবেশ করবে, অন্যথায় ফিরে যাবে।” (বুখারী ৬২৪৫, মুসলিম ২১৫৩)",'
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "ফাইজলামির সিমা থাকা দরকার!! সব পাগল চোদা সাংবাদিক!!"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "দাড়াবে?না হারিস আনিস জোসেফের মতো খালাস!!!???"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "মিথ্যা প্রচার করছ কে.....?"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "নিজে ও তার নেশাখোর ছেলে সহ অনুপ্রবেশকারী সন্ত্রাসীরা।। ২৬-১২-২১ ইং ভোট কেন্দ্রগূলোতেও বড় ধরনের সংঘাত করার প্ল্যান করা হইছে।"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "এই রকম পাগল কা..==.কা করে যাচ্ছে ততই মুগ্ধ হয়ে গেলাম"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "আজব এক ধর্ম। অবমাননার অর্থ কি ? ? কেউ বলবেন? ? মেধাহীন জাতি তা আর একবার প্রমাণ করলো ।"
    text = nmlr.text_normalizer(text)
    print(text)
    
    text = "রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম, ১২৩৪ শতাব্দীতে ¥২০৩০.১২৩৪ বিবিধ  বাকেরগঞ্জ উপজেলার প্রায় 40 ভাগের পেশাই চাষাবাদ 80 and 40 ২২"
    text = nmlr.text_normalizer(text)
    print(text)
    print("+++++++++++++++++ end random test +++++++++++++++++++++++++++")
    


import time
from module.main import Normalizer

if __name__ == "__main__":

    # # Testing Date format
    date_list = [
        "০১-এপ্রিল/২০২৩",
        "সেপ্টেম্বর ০৫ ২০২৩",
        "01-Apr/2023" "এপ্রিল ২০২৩" "2023-04-05",
        "06-04-2023",
        "04/01/2023",
        "07 April, 2023",
        "Apr 1, 2023",
        "2023/04/01",
        "01-Apr-2023",
        "01-apr/2023",
    ]
    # # # # number = "123456" or "২০২৩"
    # # number = "২০২৩"
    nrml = Normalizer()
    # # # # # print("++++++++++++++++++++ Date testing ++++++++++++++++++++++")
    # # # # print("Date format Testing : ", end ="", flush=True)
    # for date_ in date_list:
    #     start_time = time.time()
    #     print("date_:", date_)
    #     formated_date = nrml.date_format(date_, language="en")
    #     print(formated_date)
    # # print("++++++++++++++++++++ end of Date testing ++++++++++++++++++++++")

    # # print("++++++++++++++++++++ en number to bn number convert ++++++++++++++++++++++")
    # number = "1234"
    # number = nrml.number_convert(number, language="en")

    # number = nrml.number_convert(number, language="bn")

    # print("Bn Number : ", number)
    # print("++++++++++++++++++++ stop number convert ++++++++++++++++++++++")

    # # print("++++++++++++++++ Today +++++++++++++++++++++")
    # today_date = nrml.today(language="bn")
    # # today_date = nrml.today(language="en")
    # print(today_date)

    # # print("++++++++++++++++ End Today +++++++++++++++++++++")
    # # # print(today_date)
    # # print("++++++++++++++++ weekdays +++++++++++++++++++++")
    # weekdays = nrml.weekdays()
    # weekdays = nrml.weekdays(language="bn")
    # weekdays = nrml.weekdays(language="en")
    # weekdays = nrml.weekdays(day = "সোমবার")
    # weekdays = nrml.weekdays(day = "Monday")
    # print(weekdays)

    # # print("++++++++++++++++ end weekdays +++++++++++++++++++++")
    # # # print(weekdays)
    # # print("+++++++++++++++ seasons ++++++++++++++++++++++++")
    # seasons = nrml.seasons()
    # seasons = nrml.seasons(language="bn")
    # seasons = nrml.seasons(language="en")
    # seasons = nrml.seasons(seasons = "গ্রীষ্ম")
    # print(seasons)

    # # print("+++++++++++++++ end seasons ++++++++++++++++++++++++")

    # # print("+++++++++++++++++ months +++++++++++++++++++++++++++")
    # month = nrml.months()
    # month = nrml.months(month="মার্চ")
    # print(month)

    # text = "রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম, ১২৩৪ শতাব্দীতে ¥২০৩০.১২৩৪ বিবিধ  বাকেরগঞ্জ উপজেলার প্রায় 40 ভাগের পেশাই চাষাবাদ 80 and 40 ২২"

    # text = nrml.text_normalizer(text)

    # print(text)

    # input_texts = [
    #     "আমি এক দুই তিন চার পাঁচ টু থ্রি ফাইভ ছয় সেভেন এইট নাইন শূন্য আমার ফোন নাম্বার জিরো ওয়ান ডাবল সেভেন",
    #     "ওয়ান ডাবল নাইন টু",
    #     "একশ বিশ টাকা",
    #     "জিরো টু ডাবল ওয়ান",
    #     "জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন",
    #     "আমার ফোন নম্বর জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন",
    #     "ট্রিপল টু ওয়ান",
    #     "দুই হাজার চারশো বিশ",
    #     "দুই হাজার চারশ  বিশ",
    #     "হাজার বিশ",
    #     "ডাবল নাইন টু",
    #     "এক লক্ষ চার হাজার দুইশ",
    #     "এক লক্ষ চার হাজার দুইশ এক",
    #     "এক লক্ষ চার হাজার দুইশ এক টাকা এক দুই",
    #     "আমাকে এক লক্ষ দুই হাজার টাকা দেয়",
    #     "আমাকে এক লক্ষ দুই হাজার এক টাকা দেয় এন্ড তুমি বিশ হাজার টাকা নিও এন্ড এক লক্ষ চার হাজার দুইশ এক টাকা এক ডবল দুই",
    #     "ছয় হাজার বিশ",
    #     "আমার সাড়ে পাঁচ হাজার",
    #     "আমার সাড়ে তিনশ",
    #     "আড়াই হাজার",
    #     "আড়াই লক্ষ",
    #     "ডেরশ",
    #     "আমাকে ডেরশ টাকা দেয়",
    #     "সাড়ে পাঁচ কোটি টাকা",
    #     "সাড়ে 1254 টাকা",
    #     "জিরো",
    #     "একশ বিশ take একশ",
    #     "জিরো টু ডাবল ওয়ান",
    #     "জিরো টু ওয়ান ওয়ান",
    #     "থ্রি ফোর ফাইভ এইট",
    #     "একশ বিশ টাকা",
    #     "ডাবল ওয়ান ডবল টু",
    #     "জিরো ওয়ান টু",
    #     "থ্রি ফোর ফাইভ সিক্স",
    #     "সেভেন এইট নাইন টেন",
    #     "একশ দুইশ তিনশ",
    #     "চারশ পাঁচশ",
    #     "ছয়শ সাতশ",
    #     "আটশ নয়শ",
    #     "দশ তিরানব্বই",
    #     "ট্রিপল থ্রি টু",
    #     "শূন্য এক দুই তিন",
    #     "চার পাঁচ ছয় সাত",
    #     "আট নয় দশ এগারো",
    #     "বারো তেরো চৌদ্দ পনেরো",
    #     "ষোল সতেরো আঠারো উনিশ",
    #     "বিশ একুশ বাইশ তেইশ",
    #     "চব্বিশ পঁচিশ ছাব্বিশ সাতাশ",
    #     "আঠাশ ঊনত্রিশ ত্রিশ একত্রিশ",
    #     "বত্রিশ তেত্রিশ চৌত্রিশ পঁয়ত্রিশ",
    #     "ছত্রিশ সাঁইত্রিশ আটত্রিশ ঊনচল্লিশ",
    #     "চল্লিশ একচল্লিশ বিয়াল্লিশ তেতাল্লিশ",
    #     "চুয়াল্লিশ পঁয়তাল্লিশ ছেচল্লিশ সাতচল্লিশ",
    #     "আটচল্লিশ ঊনপঞ্চাশ পঞ্চাশ একান্ন",
    #     "বাহান্ন তিপ্পান্ন চুয়ান্ন পঞ্চান্ন",
    #     "ছাপ্পান্ন সাতান্ন আটান্ন ঊনষাট",
    #     "ষাট একষট্টি বাষট্টি তেষট্টি",
    #     "চৌষট্টি পঁয়ষট্টি ছেষট্টি সাতষট্টি",
    #     "আটষট্টি ঊনসত্তর সত্তর একাত্তর",
    #     "বাহাত্তর তিয়াত্তর চুয়াত্তর পঁচাত্তর",
    #     "ছিয়াত্তর সাতাত্তর আটাত্তর ঊনআশি",
    #     "আশি একাশি বিরাশি তিরাশি",
    #     "চুরাশি পঁচাশি ছিয়াশি সাতাশি",
    #     "আটাশি ঊননব্বই নব্বই একানব্বই",
    #     "বিরানব্বই তিরানব্বই চুরানব্বই পঁচানব্বই",
    #     "ছিয়ানব্বই সাতানব্বই আটানব্বই নিরানব্বই",
    #     "এক লক্ষ চার হাজার দুইশ এক টাকা এক দুই",
    #     "তিনশ পঁচিশ পাঁচশ",
    #     "তিনশ পঁচিশ পাঁচশ এক",
    #     "চা-পুন",
    #     "ওকে",
    #     "ডের আউটস্ট্যান্ডিং কত",
    #     "ডাবল",
    #     "নাইন ডাবল এইট",
    #     "দশ বারো এ এগুলা একশ একশ দুই",
    #     "এক লক্ষ তেত্রিশ চার",
    #     "আমাকে এক লক্ষ দুই হাজার এক টাকা দেয় এন্ড তুমি বিশ হাজার টাকা নিও এন্ড এক লক্ষ চার হাজার দুইশ এক টাকা এক ডবল দুই"
    #     ]
    # for i in input_texts:
    #     print("="*40)
    #     print("input : ", i)
    #     text = nrml.word2number(i)
    #     print("output : ", text)
    #     print("="*40)
    # text = "সম্মেলনটি সেপ্টেম্বর ০৫ ২০২৩ তারিখে নির্ধারিত করা হয়েছে. এপ্রিল ২,০২৩"
    # formated_date = nrml.date_extraction(text)

    # print(formated_date)

    # # text = "৫ বছরে নন-ক্যাডার পদে ৭,৪৪৭ জনকে নিয়োগের সুপারিশ করা হয়েছে 1,230"
    # text = "৫ বছরে নন-ক্যাডার পদে ৭,৪৪৭ জনকে নিয়োগের সুপারিশ করা হয়েছে"

    # text = "শীঘ্রই ভিক্টোরিয়ার এক প্রতিনিধিদল আসছেন,শিলংয়ের ব্রুকসাইড হাউসে"

    # text = "The numbers are 10 নন-ক্যাডার 20.5  30, and 40.75. সুপারিশ ২০৩০.৩০ 12 23 45"

    # bangla_sentences = [
    # "১৯৯৬ সালের ৬ তারিখে নির্ধারিত করা হয়েছে.",
    # "১৯৯৬ সালের ৬ সেপ্টেম্বর ভ্রমণ পরিকল্পনা করছি.",
    # "আমি জুলাই ২০২৩ তে একটি সমুদ্র ভ্রমণ পরিকল্পনা করছি.",
    # "সম্মেলনটি সেপ্টেম্বর ০৫ ২০২৩ তারিখে নির্ধারিত করা হয়েছে.",
    # "আমরা খ্রীষ্টমাসের জন্য ডিসেম্বর ২৫ ২০২৩ তারিখে পরিবারের সংগঠন করব.",
    # "আমার বোনের বিয়ে ১৮ মার্চ, ২০২৩ তারিখে.",
    # "আমি আগামী ১২ আগস্ট, ২০২৩ তারিখে আমার জন্মদিনে দেখা করছি.",
    # "আমরা ফেব্রুয়ারি ২০২৩ তে স্কিউইং যাব.",
    # "আমাদের কোম্পানির পিকনিকটি জুন ৩০ ২০২৩ তারিখে নির্ধারিত হয়েছে.",
    # "আমার গুরুত্বপূর্ণ পরীক্ষা ১০ মে, ২০২৩ তারিখে.",
    # "নতুন সেমিস্টার শুরু হয় জানুয়ারি ২০২৩ তারিখে.",
    # "আমরা এপ্রিল ২০২৩ তে একটি রোড ট্রিপ পরিকল্পনা করছি.",
    # "আমি অক্টোবর ২০২৩ তারিখে একটি নতুন অ্যাপার্টমেন্টে যাচ্ছি.",
    # "আপনার ক্যালেন্ডারের মার্ক নভেম্বর ০৫, ২০২৩, এটি দীপাবলি.",
    # "আমি জুলাই ২০২৩ তারিখে গরম শিবিরের জন্য উৎসাহিত.",
    # "মার্চ ২৫, ২০২৩ তারিখের কনসার্ট টিকেটগুলি দ্রুত বিক্রয় হচ্ছে.",
    # "আগস্ট ২০২৩ তারিখে একটি বারবিকিউ পার্টি আয়োজন করা হবে.",
    # "আমি মে ২০২৩ তারিখে আমার নানা-নানির কাছে যাচ্ছি.",
    # "আমি সেপ্টেম্বর ২০২৩ তারিখে একটি বিয়ে অনুষ্ঠানে যাচ্ছি.",
    # "আমরা জুন ২০২৩ তে ইউরোপে একটি ভ্রমণ পরিকল্পনা করছি.",
    # "আমার ডেন্টিস্টের অ্যাপয়েন্টমেন্ট এপ্রিল ১০, ২০২৩ তারিখে.",
    # "কোম্পানির বার্ষিক সভা ফেব্রুয়ারি ২০২৩ তারিখে হয়.",
    # "আমার অনুষ্ঠান সমাপন হয়েছে জুন ১৫, ২০২৩ তারিখে.",
    # "আমি মার্চ ২০২৩ তারিখে একটি নতুন চাকরি শুরু করছি.",
    # "আমি মে ২০২৩ তারিখে পাহাড়ে ট্রেকিং করতে যাচ্ছি.",
    # "বিদ্যালয়ের নাটকটি নভেম্বর ২০২৩ তারিখে নির্ধারিত হয়েছে.",
    # "আমরা অক্টোবর ২০২৩ তার",
    # "১৯৯৬ সালের ৬ সেপ্টেম্বর",
    # "২০৩০ সালের ৬ সেপ্টেম্বর",
    # "১৯৯৬ সালের৬ তারিখে নির্ধারিত করা হয়েছে.",
    # "১৯৯৬সালের ৬ সেপ্টেম্বররণ ভ্রমণ পরিকল্পনা করছি ২০৩০সালের ৬সেপ্টেম্বর",
    # "এয়ারলাইনসসহ তিনটি এয়ারলাইনসের ৫১টি ফ্লাইটে মো: ২০ হাজার ২৯১ জন হজযাত্রী সৌদি আরবে গেছেন। এ বছর হজ করতে ৮৫ হাজার ২৫৭ জনের সৌদি আরবে যাওয়ার কথা রয়েছে।",
    # " সাঃ কিমি হাফিক বিডিটি এয়ারলাইনসসহ তিনটি এয়ারলাইনসের ৫১টি কিমি. ফ্লাইটে মোট ২০ হাজার ২৯১ জন হজযাত্রী সৌদি আরবে গেছেন। এ বছর হজ করতে ৮৫ হাজার ২৫৭ জনের সৌদি আরবে যাওয়ার কথা রয়েছে। ৫ম",
    # "প্রদর্শনীটি চলার কথা ছিল ১২ মে পর্যন্ত, তবে দর্শকদের ব্যাপক আগ্রহের কারণে তিন দিন সময় বাড়িয়ে ১৫ মে পর্যন্ত করা হয়েছিল ১৫.১৫",
    # "The numbers are 10 নন-ক্যাডার 20.5  30, and 40.75. সুপারিশ ২০৩০.৩০ 12 23 45",
    # "[১৯৯৬]-সালের-৬:তারিখে-নির্ধারিত করা হয়েছে"
    # ]

    # # solving steps is have number need to give extra space both side
    # issue = [
    # # "[১৯৯৬]-সালের-৬:তারিখে-নির্ধারিত করা হয়েছে.",
    # # "১৯৯৬সালের ৬ সেপ্টেম্বররণ ভ্রমণ পরিকল্পনা করছি ২০৩০সালের ৬সেপ্টেম্বর",
    # # "আজকের তাপমাত্রা ৪৪°F",
    # # "আজকের তাপমাত্রা ৪৪°C",
    # # "যেমন ১৯৬১ সালে দেশটির তৎকালীন প্রেসিডেন্ট ডোয়াইট ডি আইজেনহাওয়ার শিক্ষা খাতে সামরিক শিল্পের প্রবেশের বিপদ নিয়ে সতর্ক করেছিলেন।",
    # # "১৯৯৬সালের ৬ সেপ্টেম্বররণ ভ্রমণ পরিকল্পনা করছি ২০৩০সালের ৬সেপ্টেম্বর",
    # "উদাহরণস্বরূপ, আপনার মোটরযানের রেজিস্ট্রেশন নাম্বার ঢাকা মেট্রো-গ-12-1212 এবং টাকা জমা রশিদের ট্রানজেকশন নাম্বার 2001011325989"
    # ]

    issue = [
        # "[১৯৯৬]-সালের-৬:তারিখে- ১৯৯৬—৯৯ নির্ধারিত করা হয়েছে. ১৯৯৬-৯৯",
        # "১৯৮৭-র ১৯৯৫ সালে",
        # "দয়া !@করে #পবিত্র &)কুরআনুল *কারিম বলেন,,,,পবিত্র কথাটা অবশ্যই বলবেন,,, প্লিজ",
        # "১৯৯৬সালের ৬ সেপ্টেম্বররণ ভ্রমণ পরিকল্পনা ৬ সেপ্টেম্বর করছি ২০৩০সালের ২সেপ্টেম্বর",
        # " দুর্নীতির মাধ্যমে অর্জিত আয়ের উৎস গোপনের জন্য নিজের নামসহ তাঁর আত্মীয়স্বজনের নামে ৭০০০টির বেশি হিসাব খোলেন।"
        # "১৯৯৬ সালের নির্ধারিত করা হয়েছে. ১৯৯৬-৯৬",
        # "১৯৯৬সালের ৬ সেপ্টেম্বররণ ভ্রমণ পরিকল্পনা করছি ২০৩০সালের ৬সেপ্টেম্বর দুর্নীতির মাধ্যমে অর্জিত আয়ের উৎস গোপনের জন্য নিজের নামসহ তাঁর আত্মীয়স্বজনের নামে ৭০০০টির বেশি হিসাব খোলেন।",
        # "আজকের তাপমাত্রা৪৪°F",
        # "আজকের তাপমাত্রা ৪৪°C",
        # "যেমন ১৯৬১ সালে দেশটির তৎকালীন প্রেসিডেন্ট ডোয়াইট ডি আইজেনহাওয়ার শিক্ষা খাতে সামরিক শিল্পের প্রবেশের বিপদ নিয়ে সতর্ক করেছিলেন।",
        # "১৯৯৬সালের ৬ সেপ্টেম্বররণ ভ্রমণ পরিকল্পনা করছি ২০৩০সালের ৬সেপ্টেম্বর",
        # " বাংলাদেশের পাসপোর্টের অভ্যন্তরে ইস্যু করানোর সময়ে নিয়মিত, জরুরী, এবং অতীব জরুরী বিতরণের ফি নিম্নলিখিত: নিয়মিত বিতরণ (48 পৃষ্ঠা এবং 10 বছর মেয়াদ) পাসপোর্ট ফি: ৪,০২৫ টাকা জরুরী বিতরণ (48 পৃষ্ঠা এবং 10 বছর মেয়াদ) পাসপোর্ট ফি: ৬,৩২৫ টাকা অতীব জরুরী বিতরণ (48 পৃষ্ঠা এবং 10 বছর মেয়াদ) পাসপোর্ট ফি: ৮,৬২৫ টাকা নিয়মিত বিতরণ (64 পৃষ্ঠা এবং 5 বছর মেয়াদ) পাসপোর্ট ফি: ৫,৭৫০ টাকা জরুরী বিতরণ (64 পৃষ্ঠা এবং 5 বছর মেয়াদ) পাসপোর্ট ফি: ৮,০৫০ টাকা অতীব জরুরী বিতরণ (64 পৃষ্ঠা এবং 5 বছর মেয়াদ) পাসপোর্ট ফি: ১০,৩৫০ টাকা",
        # "বাংলাদেশের পাসপোর্টের অভ্যন্তরে ইস্যু করানোর সময় 10/12/1956",
        # "৬/৯/১৯৯৬ বাংলাদেশের পাসপোর্টের অভ্যন্তরে ইস্যু করানোর সময় 10/12/24",
        # "ডিজিটাল রেজিস্ট্রেশন সার্টিফিকেট সংক্রান্ত যোগাযোগ করতে হলে 01790-111111 অথবা 01790-540210 নম্বরে যোগাযোগ করতে হবে +8801790540211",
        # "ডিজিটাল রেজিস্ট্রেশন সার্টিফিকেট সংক্রান্ত যোগাযোগ করতে হলে ০১৭৯০-১১১১১১ অথবা ০১৭৯০-৫৪০২১০ নম্বরে যোগাযোগ করতে হবে ০১৭৯০৫৪০২১১",
        # "রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম, ১২৩৪ শতাব্দীতে ¥২০৩০.১২৩৪ বিবিধ  বাকেরগঞ্জ উপজেলার প্রায় 40 ভাগের পেশাই চাষাবাদ 80 and 40 ২২"
        # "অর্থাৎ, আবেদনকারীর বয়স ১ বছর হলে জন্ম নিবন্ধন ফি ১০ টাকা, বয়স ২ বছর হলে ২০ টাকা, বয়স ৫ বছর হলে ৫০ টাকা, এবং বয়স ১০ বছর হলে ১০০ টাকা হবে",
        # "বাংলাদেশের পাসপোর্টের ৪৮ পৃষ্ঠা এবং ১০ বছর মেয়াদের সহ পাসপোর্টের জন্য নিয়মিত বিতরণে ফি ৫,৭৫০ টাকা, জরুরী বিতরণে ফি ৮,০৫০ টাকা, এবং অতীব জরুরী বিতরণের ফি ১০,৩৫০ টাকা",
        "১৯৯৬ টাকা বাংলাদেশের পাসপোর্টের ৬৪ পৃষ্ঠা এবং ৫ বছর মেয়াদের সহ পাসপোর্টের জন্য নিয়মিত বিতরণে ফি ৬,৩২৫ টাকা, জরুরী বিতরণে ফি ৮,৬২৫.৬২৫ টাকা, এবং অতীব জরুরী বিতরণের ফি ১২,০৭৫ টাকা"
        # "1995 সাল  ,  কালো রাত। বুধবার থেকে দুটি আবাসিক এলাকার ৯টি ব্লকে ৯টি দল পাহারা দিচ্ছে। মোসা. রশিদের "
        # "১৯৫৪ সালের রাহিম ক্লাস ওয়ান এ ১ম, এন্ড বাসার ক্লাস এ ৩৩ তম, সে জন্য ২০৩০ শতাব্দীতে ¥২০৩০.১২৩৪ দিতে হয়েছে",
        # "উদাহরণস্বরূপ, আপনার মোটরযানের রেজিস্ট্রেশন নাম্বার ঢাকা মেট্রো-গ-12-1212 এবং টাকা জমা রশিদের ট্রানজেকশন নাম্বার 2001011325989",
        # "Examples: 2024-05, 1254, 1195–05, 1995—05, 2024―05, 01775-937895"
        """আমি যে যে সার্ভিস দিয়ে থাকি 
        ১.০ মুড়ি বিক্রি 
        ২. চাল
        ৩. ডাল 
        4.1 ডাল"""
    ]

    for i in issue:
        print("input : ", i)
        text = nrml.text_normalizer(i)
        print("output : ", text)
        print("=" * 2)
        print(nrml.text_diff(i, text))
        # print()
        print("+" * 40)

    n = "জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন"

    number_string = nrml.word2number(n)
    print(number_string)



import pandas as pd
import time
import csv
import difflib

import time
from module.main import Normalizer

nrml = Normalizer()


def read_test_file(file_path, lj_speach=True):
    """
    read txt file
    """
    with open(file_path, "r") as file:
        # Read all lines from the file
        data = [i for i in file.read().split("\n") if i]

    if lj_speach:
        data_list = []
        for i in data:
            audio, text = i.split("|")[0], i.split("|")[1:]
            data_list.append((" ".join(text)).strip())
    else:
        data_list = data
    return data_list


def read_excel_file(file_path, sheet_names):

    # Read the specified sheets from the Excel file
    sheets = pd.read_excel(file_path, sheet_name=sheet_names)

    # If you need the DataFrames in a list instead of a dictionary
    sheets_list = [sheets[sheet] for sheet in sheet_names]

    # Display the DataFrames
    text_list = []
    for i, df in enumerate(sheets_list):
        df_filled = df.fillna("")
        # print(f"\nData from {sheet_names[i]}:")
        texts = df_filled.Sentences.tolist()
        text_list.extend(texts)
    text_list = [i for i in text_list if i]
    return text_list


def csv_log_generation(data, header, output_path="report/pybangla_report_csv.csv"):
    with open(output_path, mode="w", newline="") as file:
        writer = csv.writer(file)
        writer.writerow(header)  # Write the header row
        writer.writerows(data)  # Write the data rows`


def find_changes(string1, string2):
    # Split the strings into words
    words1, words2 = string1.split(), string2.split()
    # Use difflib to find differences
    diff = difflib.ndiff(words1, words2)
    added_chunk, added_chunks, removed_chunk, removed_chunks = [], [], [], []
    for change in diff:
        if change.startswith("+ "):
            added_chunk.append(change[2:])
            if removed_chunk:
                removed_chunks.append(" ".join(removed_chunk))
                removed_chunk = []
        elif change.startswith("- "):
            removed_chunk.append(change[2:])
            if added_chunk:
                added_chunks.append(" ".join(added_chunk))
                added_chunk = []
        else:
            if added_chunk:
                added_chunks.append(" ".join(added_chunk))
                added_chunk = []
            if removed_chunk:
                removed_chunks.append(" ".join(removed_chunk))
                removed_chunk = []
    # Append any remaining chunks
    if added_chunk:
        added_chunks.append(" ".join(added_chunk))
    if removed_chunk:
        removed_chunks.append(" ".join(removed_chunk))

    return removed_chunks, added_chunks


# # Example usage
# string1 = "উদাহরণস্বরূপ, আপনার মোটরযানের রেজিস্ট্রেশন নাম্বার ঢাকা মেট্রো-গ-12-1212 এবং টাকা জমা রশিদের ট্রানজেকশন নাম্বার 2001011325989"
# string2 = "উদাহরণস্বরূপ, আপনার মোটরযানের রেজিস্ট্রেশন নাম্বার ঢাকা মেট্রো গ-বারো-বারোবারো এবং টাকা জমা রশিদের ট্রানজেকশন নাম্বার দুই লক্ষ এক শত এক কোটি তেরো লক্ষ পঁচিশ হাজার নয় শত ঊননব্বই"

# string1= "৬/৯/১৯৯৬ বাংলাদেশের পাসপোর্টের অভ্যন্তরে ইস্যু করানোর সময় 10/12/24"
# string2 = "ছয় সেপ্টেম্বর উনিশশো ছিয়ানব্বই বাংলাদেশের পাসপোর্টের অভ্যন্তরে ইস্যু করানোর সময় দশ বারো চব্বিশ"


if __name__ == "__main__":

    # file_path = "./test_data/evaluation_data.xlsx"
    file_path = "./test_data/metadata_udoy_lj.txt"
    # sheet_names = [
    #     'saiful',
    # ]
    # texts = read_excel_file(file_path, sheet_names)
    texts = read_test_file(file_path)
    # print(len(texts))

    index = 0
    s_time = time.time()
    header = ["Input_Text", "Process_Text", "Status", "Before Replace", "After Replace"]
    process_list = []
    for text in texts:
        print("input  : ", index, text)
        p_text = nrml.text_normalizer(text)
        print("process text : ", p_text)
        removed_chunks, added_chunks = find_changes(text, p_text)
        # Output the results
        print(f"Removed chunks: {removed_chunks}")
        print(f"Added chunks: {added_chunks}")
        process_list.append(
            [text, p_text, text == p_text, removed_chunks, added_chunks]
        )
        index += 1
    print("time : ", time.time() - s_time)
    csv_log_generation(process_list, header)



import re
import datetime
import string
from .config import Config as cfg
from num2words import num2words
from .date_extractor import DateExtractor
from fuzzywuzzy import fuzz

# from bnemo import Translator
from .phone_number_extractor import PhoneNumberExtractor


dt = DateExtractor()
pne = PhoneNumberExtractor()
data = cfg.data

_abbreviations = cfg._abbreviations
_symbols = cfg._symbols
_ordinal_re = cfg._ordinal_re
_whitespace_re = cfg._whitespace_re
_currency = cfg._currency
_punctuations = cfg._punctuations

english_digits = cfg._bangla2english_digits_mapping
bangla_numeric_words = cfg._bangla_numeric_words

_STANDARDIZE_ZW = cfg._STANDARDIZE_ZW
_DELETE_ZW = cfg._DELETE_ZW


class NumberParser:
    def __init__(self):
        self.english_digits = english_digits
        self.bangla_numeric_words = bangla_numeric_words
        self.en_regex = cfg.en_regex
        self.bn_regex = cfg.bn_regex

    def is_english_digit_string(self, s):
        # Check if all characters in the string are digits (0-9)
        return all(char.isdigit() for char in s)

    def contains_only_english(self, input_string):
        # Check if all characters in the string are English (ASCII) characters
        return all(ord(char) < 128 for char in input_string)

    def number_to_words_converting_process(self, number_string: str, lang="bn"):
        number_string = number_string.strip()
        num = int(
            "".join(
                [
                    self.english_digits[bangla_digit] if lang == "bn" else bangla_digit
                    for bangla_digit in number_string
                ]
            )
        )
        try:
            eng_in_num_to_words = num2words(num, lang="en_IN")
            if lang == "bn":
                bangla_num_to_words_list = [
                    self.bangla_numeric_words[word]
                    for word in eng_in_num_to_words.replace(",", " ")
                    .replace(" and ", " ")
                    .split()
                ]
            else:
                bangla_num_to_words_list = [
                    word
                    for word in eng_in_num_to_words.replace(",", " ")
                    .replace(" and ", " ")
                    .split()
                ]
            return " ".join(bangla_num_to_words_list)
        except Exception as e:
            print(e)
            return

    def number_to_words(self, number: str, chunk_millions=7):

        en_extraction = list(re.finditer(self.en_regex, number, re.UNICODE))
        number = number[::-1]
        chunks = [
            number[i : i + chunk_millions]
            for i in range(0, len(number), chunk_millions)
        ]
        chunks = [c[::-1] for c in chunks]
        chunks = chunks[::-1]
        if en_extraction:
            number = " crore ".join(
                [
                    self.number_to_words_converting_process(chunk, lang="en")
                    for chunk in chunks
                ]
            )
            number = number.replace("zero", "")
        else:
            number = " কোটি ".join(
                [
                    self.number_to_words_converting_process(chunk, lang="bn")
                    for chunk in chunks
                ]
            )
            # print(number)
            number = number.replace("শূন্য", "")

        return " ".join(number.split())

    def digit_number_to_digit_word(self, number, language="bn"):

        number = re.sub(_whitespace_re, " ", number)
        s_n = ""
        for i in number:
            # print("language : ", language, i)

            n = data[language]["number_mapping"][i]
            s_n += " " + n
        return s_n.strip()

    def year_in_number(self, year_in_number: str, language="bn"):
        """Converts a Bangla year in numeric form to literal words.

        Args:
            number_string: Bangla year in numbers as string. Example: "১৯৯৪"

        Returns:
            Bangla year in words. Example: "উনিশশো চুরানব্বই"

        """
        english_status = self.contains_only_english(year_in_number)
        if english_status:
            year_in_number = "".join(
                [cfg._english2bangla2_digits_mapping[i] for i in year_in_number]
            )
        if language == "bn":
            mid_text = "শো "
        else:
            mid_text = " century "

        if (len(year_in_number) == 4 and year_in_number[1] != "০") or len(
            year_in_number
        ) == 3:
            # print("year in ")

            if year_in_number[1] != "0":
                year_str = self.number_to_words(year_in_number)
            return (
                self.number_to_words(year_in_number[:-2])
                + mid_text
                + self.number_to_words(year_in_number[-2:])
            )
        else:
            # print("+++++++++ else+++++++++++++++")
            return self.number_to_words(year_in_number)

    def _replace_starting_zero(self, month):
        """
        Normalize string which start zero first

        """
        if month[0] == "0" or month[0] == "০":
            return month[1:]
        return month

    def _digit_converter(self, number, language="bn"):
        """
        convert the digit En to Bn or Bn to En

        """
        # print("number  : ", number, language)

        if language == "en":
            extracted_number = list(re.finditer(self.bn_regex, str(number), re.UNICODE))
            if extracted_number:
                # print("language", number)
                # [i[1] for i in number if i[0]=="0"]
                if number[0] == "0":
                    number = number[1:]
                number = "".join(
                    [
                        cfg._bangla2english_digits_mapping[i.replance(",", "")]
                        for i in number
                        if i[0] == "0"
                    ]
                )
                # print("extracted : ", number)
        c_number = ""
        for n in number:
            n = n.replace(",", "")
            if n:
                if n in data[language]["number"]:
                    c_number += n
                else:
                    if n.strip() in data[language]["digits_mapping"]:
                        b_n = data[language]["digits_mapping"][n.strip()]
                        c_number += b_n
        return c_number

    def get_weekday(self, date_: list = [], language="bn"):
        """
        Get weekday name Bangla or English

        """

        # print("date_", date_)
        try:
            if date_[0] is None or date_[1] is None or date_[2] is None:
                return None
            elif date_[1].isdigit():
                if int(date_[1]) > 12:
                    return None
            d, y = list(re.finditer(self.bn_regex, str(date_[0]), re.UNICODE)), list(
                re.finditer(self.bn_regex, str(date_[2]), re.UNICODE)
            )

            if d:
                date_[0] = self._digit_converter(date_[0], language="bn")
            if y:
                date_[2] = self._digit_converter(date_[2], language="bn")

            current_date_object = datetime.datetime(
                int(date_[2]), int(date_[1]), int(date_[0])
            )
            if language in data:
                weekday = data[language]["weekdays"][current_date_object.weekday()]
            else:
                print("language not handel")
                weekday = ""
        except:
            weekday = None
        # print("weekday : ", weekday)
        return weekday

    def search_month(self, search_key, language="bn"):
        """
        Search for a month or month abbreviation in the month_data dictionary.

        Args:
            search_key (str): The month or its abbreviation to search for.
            language (str, optional): Language identifier ("bn" for Bengali, "en" for English).
                Defaults to "bn".

        Returns:
            list: A list containing additional information about the month if found,
            formatted based on the specified language.
            The list contains [month_name, season_name, number_of_days].
            If the month or abbreviation is not found, returns [None, None, None].
        """
        try:
            search_key = int(self._replace_starting_zero(search_key)) - 1
            month = data[language]["months"][search_key]
            seasons = data[language]["seasons"][search_key // 2]
            option_name = data[language]["option_name"][search_key]
        except:
            month, option_name, seasons = None, None, None
        return [month, option_name, seasons]

    def find_word_index(self, text: str, word: str) -> list:
        """
        Word spanning position
        """
        start = text.find(word)
        end = start + len(word)
        return [start, end]

    def replace_text_at_position(
        self, text: str, replacement: str, start_pos: int, end_pos: int
    ) -> str:
        """
        Replance text using text position

        """
        rep_text = text[:start_pos] + replacement + text[end_pos:]
        return rep_text

    def fraction_number_conversion(self, number, language="bn"):
        n_n = ""
        for i in number:
            if i in cfg._english2bangla2_digits_mapping:
                n_n += cfg._english2bangla2_digits_mapping[i]
            else:
                n_n += i

        s_m = n_n.split(".")
        before_dot_word, after_dot_word = self.number_to_words(
            s_m[0]
        ), self.digit_number_to_digit_word(s_m[1], language=language)

        if len(after_dot_word):
            word = before_dot_word + " দশমিক " + after_dot_word
            return word
        return before_dot_word

    def check_comma_dot_dari(self, p):
        l_p = [",", ".", "।"]
        if p in l_p:
            return True
        return False

    def number_processing(self, text):
        pattern = r"[\d,\.]+"
        matches = re.findall(pattern, text)
        sorted_matches = sorted(matches, key=len, reverse=True)
        for n in sorted_matches:
            p_status = self.check_comma_dot_dari(n)
            if p_status:
                text = text.replace(n, " " + n + " ")
                # print("p_status : ", text)
            else:
                status = self.contains_only_english(n)
                m_re = n.replace(",", "")
                if status:

                    if "." in m_re:
                        bn_m = self.fraction_number_conversion(m_re)
                    else:
                        bn_m = self.number_to_words(self._digit_converter(m_re))
                    text = text.replace(n, " " + str(bn_m) + " ")
                else:
                    if "." in m_re:
                        bn_m = self.fraction_number_conversion(m_re, language="bn")
                    else:
                        bn_m = self.number_to_words(m_re)
                    # print("else : bn_m ", n, bn_m)
                    text = text.replace(str(n), " " + str(bn_m) + " ")
        return text


class DateParser:
    def __init__(self):
        self.samples = cfg.samples

        self.npr = NumberParser()

    def data_splitter(self, date_string):
        """ """
        separator_pattern = "|".join(map(re.escape, self.samples))
        return re.split(separator_pattern, date_string)

    def month_convert_to_number(self, month):
        """ """
        index = None
        # print("month", month)
        key = month.lower().strip()
        if key in data["en"]["months"]:
            index = data["en"]["months"].index(key) + 1
        elif key in data["bn"]["months"]:
            index = data["bn"]["months"].index(key) + 1
        elif key in data["bn"]["option_name"]:
            index = data["bn"]["option_name"].index(key) + 1
        elif key in data["en"]["option_name"]:
            index = data["en"]["option_name"].index(key) + 1
        elif key in data["en"]["number"]:
            index = data["en"]["number"].index(key) + 1
        elif key in data["bn"]["number"]:
            index = data["bn"]["number"].index(key) + 1
        else:
            key = key.strip()
            if key[-1] in string.punctuation:
                index = key[:-1]
        return index

    def format_non_punctuation(self, split_date):
        """ """
        if len(split_date[0]) == 8:
            if int(split_date[0][4:6]) <= 12:
                year, month, day = (
                    split_date[0][:4],
                    split_date[0][4:6],
                    split_date[0][6:],
                )
            else:
                year, month, day = (
                    split_date[0][4:],
                    split_date[0][2:4],
                    split_date[0][:2],
                )
            return [day, month, year]
        else:
            print("This date format is not handled yet")
        return None

    def get_day_and_month(self, year_idx, idx, date_list):
        """ """
        if year_idx == 0:
            return self.get_day_and_month_helper(idx, date_list, 1, 2)
        elif year_idx == 2:
            return self.get_day_and_month_helper(idx, date_list, -1, -2)
        else:
            print("Date format not handled yet")
        return None, None

    def get_day_and_month_helper(self, idx, date_list, offset1, offset2):
        """ """
        if date_list[idx + offset1].isdigit() and date_list[idx + offset2].isdigit():
            return date_list[idx + offset2], date_list[idx + offset1]
        elif (
            not date_list[idx + offset1].isdigit()
            and date_list[idx + offset2].isdigit()
        ):
            return date_list[idx + offset2], self.month_convert_to_number(
                date_list[idx + offset1]
            )
        elif (
            date_list[idx + offset1].isdigit()
            and not date_list[idx + offset2].isdigit()
        ):
            return date_list[idx + offset1], self.month_convert_to_number(
                date_list[idx + offset2]
            )
        else:
            print("Date format not handled yet")
        return None, None

    def get_date_indexes(self, date_list):
        """
        Get Date index
        """

        # print(date_list)
        day, month, year = None, None, None
        for idx, elem in enumerate(date_list):
            if elem.isdigit() and len(elem) == 4:
                year_idx = idx
                year = date_list[idx]
                # print(year)
                day, month = self.get_day_and_month(year_idx, idx, date_list)
                # print(day, month)
        return [day, month, year]

    def date_processing(self, date_, slash_status=True, language="bn"):

        # print(date_)
        if isinstance(date_, list):
            if len(date_):
                formatted_date = date_
        else:
            split_date = self.data_splitter(date_)
            # print("split_date : ", split_date)
            split_date = [i for i in split_date if i]
            # print("split_date : ", split_date)
            if len(split_date) == 2:
                adding_date = ["1"] if language == "en" else ["১"]
                split_date = adding_date + split_date

            if len(split_date) == 1:
                formatted_date = self.format_non_punctuation(split_date)
            else:
                # print("hello")
                formatted_date = self.get_date_indexes(split_date)

                # print("formatted_date :", formatted_date)

        if (
            formatted_date[0] is None
            and formatted_date[1] is None
            and formatted_date[2] is None
        ):
            current_date_object = datetime.date.today()
            formatted_date = [
                current_date_object.day,
                current_date_object.month,
                current_date_object.year,
            ]

        elif formatted_date[1] == None:
            for i in split_date:
                if formatted_date[0] == i or formatted_date[2] == i:
                    continue
                m_numeric = self.month_convert_to_number(i)
                if m_numeric:
                    formatted_date[1] = cfg._english2bangla2_digits_mapping[
                        str(m_numeric)
                    ]

        # print("final formated date : ", formatted_date)

        if (
            formatted_date[0] is not None
            and formatted_date[1] is not None
            and formatted_date[2] is not None
        ):
            weekday = self.npr.get_weekday(formatted_date, language)
        else:
            weekday = None

        if formatted_date[0] is None:
            day = None
            txt_date = None
        else:
            day = self.npr._digit_converter(str(formatted_date[0]), language)
            txt_date = self.npr.number_to_words(day)
        if formatted_date[1] is None:
            month = [None, None, None]
            m_n = None
        else:
            m_n = self.npr._digit_converter(str(formatted_date[1]), language)
            month = self.npr.search_month(str(formatted_date[1]), language)
        if formatted_date[2] is None:
            year = None
            txt_year = None
        else:
            year = self.npr._digit_converter(str(formatted_date[2]), language)
            txt_year = self.npr.year_in_number(year, language=language)

        return {
            "date": day,
            "month": m_n,
            "year": year,
            "txt_date": txt_date,
            "txt_month": month[0],
            "txt_year": txt_year,
            "weekday": weekday,
            "ls_month": month[1],
            "seasons": month[2],
        }


class TextParser:

    def __init__(self):
        self.year_patterns = ["সালের", "সালে", "শতাব্দী", "শতাব্দীর", "শতাব্দীতে"]
        self.year_pattern = (
            r"(?:\b|^\d+)(\d{4})\s*(?:সালে?র?|শতাব্দী(?:র)?|শতাব্দীতে|এর)+"
        )
        self.currency_pattern = (
            r"(?:\$|£|৳|€|¥|₹|₽|₺)?(?:\d+(?:,\d{3})*(?:\.\d+)?|\d+(?:\.\d+)?)"
        )
        self.npr = NumberParser()
        self.dp = DateParser()

    def collapse_whitespace(self, text):
        text = re.sub(_whitespace_re, " ", text)
        text = re.sub(r"\s*,\s*", ", ", text)
        return text

    def phone_number_processing(self):
        pass

    def exception_year_processing(self, text):

        # if len(text
        # s+(\d{4}(?:-|–|—|―)\d{2})\s+
        # _year_with_hyphen = re.findall(r'(\d{4}(?:-|–|—|―)\d{2})', text)
        # print(_year_with_hyphen)
        # _year_with_hyphen = re.findall(r'\s+(\d{4}(?:-|–|—|―)\d{2})\s+', text)
        _year_with_hyphen = re.findall(r"\b(\d{4}[-–—―]\d{2})\b", text)
        replce_map = {}
        for year in _year_with_hyphen:
            rep_year = year.replace("–", "-")
            rep_year = rep_year.replace("—", "-")
            rep_year = rep_year.replace("―", "-")
            four_digit_year, two_digit_year = rep_year.split("-")

            en_status_four = self.npr.is_english_digit_string(four_digit_year)
            en_status_two = self.npr.is_english_digit_string(two_digit_year)

            if en_status_four:
                four_digit_year = self.npr._digit_converter(
                    four_digit_year, language="bn"
                )
            if en_status_two:
                two_digit_year = self.npr._digit_converter(
                    two_digit_year, language="bn"
                )

            rep_year = (
                self.npr.year_in_number(four_digit_year)
                + " "
                + self.npr.number_to_words(two_digit_year)
            )
            text = text.replace(year, rep_year)
        return text

    def unwanted_puntuation_removing(self, text):

        # https://stackoverflow.com/questions/63256077/how-to-remove-redundant-punctuations-keep-only-the-first-one-in-text
        def my_replace(match):
            match = match.group()
            return match[0] + (" " if " " in match else "")

        _redundent_punc_removal = r"[!\"#$%&\'()*+,\-.\/:;<=>?@\[\\\]^_`।{|}~ ]{2,}"
        _remove_hyphen_slash = r"(?<!\d)[-/](?!\d)"
        _remove_comma = r"(?<=\d),(?=\d)"
        _remove_space_in_punctuations = r"(?<=[^\w\s])\s+(?=[^\w\s])"

        text = _STANDARDIZE_ZW.sub("\u200D", text)
        text = re.sub(r"\u200d", "", text)
        text = _DELETE_ZW.sub("", text)

        text = text.replace("'র", " এর")
        text = text.replace("-র", " এর")
        text = text.replace("\uf038", " ")
        text = text.replace("°F", "° ফারেনহাইট")
        text = text.replace("° F", "° ফারেনহাইট")
        text = text.replace("°C", "° সেলসিয়াস")
        text = text.replace("° C", "° সেলসিয়াস")
        text = text.replace("-সালের", " সালের")
        text = text.replace("-সাল", " সাল")
        text = re.sub(_remove_space_in_punctuations, "", text)
        text = re.sub(
            _redundent_punc_removal, my_replace, text, 0
        )  # only keep the first punctuation
        text = re.sub(_remove_comma, "", text)
        text = re.sub(_remove_hyphen_slash, " ", text)
        translation_table = str.maketrans(_punctuations)
        text = text.translate(translation_table)
        return text

    def expand_symbols(self, text, lang="bn"):
        for key, replacement in _symbols[lang]:
            text = text.replace(key, replacement)
        return text.strip()

    def expand_abbreviations(self, text, lang="bn"):
        for key, replacement in _abbreviations[lang]:
            text = text.replace(key, replacement)
        return text

    def expand_position(self, text, lang="bn"):
        """
        Replace :
        রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম -> রাহিম ক্লাস ওয়ান এ প্রথম, এগারোতম বাইশতম তেত্রিশতম

        """
        for regex, replacement in _ordinal_re[lang]:
            text = re.sub(regex, replacement, text)
        if lang == "bn":
            matches = re.findall(r"(\d+)(?:\s*)(?:তম)", text)
            if matches:
                for i in matches:
                    word = self.npr.number_to_words(i)
                    text = text.replace(i + "তম", word + " তম")
                    text = text.replace(i + " তম", word + " তম")
        return text

    def extract_year_blocks_with_positions(self, text):
        matches = re.finditer(self.year_pattern, text)

        results = []
        for match in matches:
            block = match.group(0)
            start_pos = match.start()
            end_pos = match.end()
            results.append((block, start_pos, end_pos))

        return results

    def year_formation(self, text):

        for i in self.year_patterns:
            # print(i)
            if i in text:
                text = text.replace(i, " " + i)
        text = self.collapse_whitespace(text)
        matches = self.extract_year_blocks_with_positions(text)[::-1]

        # print(" matches : ", matches)
        """
        Need to correct year format extraction
        """
        for i in matches:
            # print("match", i)
            extract_year = [y for y in i[0].split(" ") if y.isnumeric() and len(y) == 4]
            # print(extract_year)
            start_pos, end_pos = i[1], i[1] + len(extract_year[0])
            process_year = self.npr.year_in_number(extract_year[0])
            text = text[:start_pos] + process_year + text[end_pos:]
        return text

    def extract_currency_amounts(self, text):

        matches = re.findall(self.currency_pattern, text)
        pattern = r"[৳$£€¥₹₽₺]"

        sorted_matches = sorted(matches, key=len, reverse=True)

        for m in sorted_matches:
            currency = re.findall(pattern, m)
            if currency:
                n_m = m.replace(currency[0], "")
                n_m = n_m.replace(",", "")
                language = "en" if self.npr.contains_only_english(n_m) else "bn"
                if "." in n_m:
                    word = self.npr.fraction_number_conversion(n_m, language=language)
                    r_word = " " + word + " " + _currency[currency[0]] + " "
                    text = text.replace(m, r_word)
                else:
                    word = self.npr.number_to_words(n_m)
                    n_word = " " + word + " " + _currency[currency[0]] + " "
                    text = text.replace(m, n_word)
        return text

    def matching_similariy_of_months(self, input_word):
        month_name = (
            data["en"]["months"]
            + data["bn"]["months"]
            + data["en"]["option_name"]
            + data["bn"]["option_name"]
        )
        similarity_threshold = 90  # Adjust this threshold as needed
        similar_months = []
        status = False
        for month in month_name:
            similarity_score = fuzz.partial_ratio(input_word, month)
            if similarity_score >= similarity_threshold:
                similar_months.append((month, input_word, similarity_score))

        # print("Similar months:")
        for month, input_word, similarity_score in similar_months:
            # print(f"{input_word} {month}: Similarity Score = {similarity_score}")
            status = True
        # return True
        if similar_months:
            sorted_similar_months = sorted(
                similar_months, key=lambda x: x[2], reverse=True
            )
            # print("+++++++", sorted_similar_months)
            return status, (sorted_similar_months[0][0], sorted_similar_months[0][1])
        return status, (None, None)

    def date_formate_validation(self, date, text):

        # print("data format validation : ", date,"====", text)
        n_data = date.strip().split(" ")
        # print("n_data : ", n_data)
        for n_d in n_data:
            status, text_replacer = self.matching_similariy_of_months(n_d)
            if status:
                for t in text_replacer:
                    text = text.replace(t[0], t[1])
                return status, text
            if n_d in self.year_patterns:
                return True, text
        return False, text

    def add_spaces_to_numbers(self, text):

        # print("text : ", text)
        # Define a regular expression pattern to match both Bangla and English digits without spaces around them
        pattern = r"(?<![০-৯0-9])[\u09E6-\u09EF0-9]+(?![০-৯0-9])"
        # Use re.sub to find and replace matches with spaces around them
        result = re.sub(pattern, lambda x: " " + x.group(0) + " ", text)
        # print("result : ", result)
        result = " ".join([i for i in result.split(" ") if i.strip()]).strip()
        return result

    def extract_year(self, text):
        pattern = re.findall(self.year_pattern, text)
        for p in pattern:
            text = text.replace(p, f" {p} ")
        return text

    def check_date_format(self, date_string):
        # Define the regex pattern for English and Bangla digits with slashes or hyphens
        pattern = r"^[\d০-৯]{1,2}[-/][\d০-৯]{1,2}[-/][\d০-৯]{2}([\d০-৯]{2})?$"

        # Check if the date string matches the pattern
        if re.match(pattern, date_string):
            return True
        else:
            return False

    def check_date_format_exception_case(self, date_string):
        if not self.check_date_format(date_string):
            return None

        # Split the date string by either '/' or '-'
        parts = re.split(r"[-/]", date_string)

        return parts

    def english_date_to_bangla_date(self, date_list):

        bn_data_list = []
        for d_l in date_list:
            en_digits_status = self.npr.is_english_digit_string(d_l)
            if en_digits_status:
                d_character = []
                for en_d in d_l:
                    if en_d in cfg._english2bangla2_digits_mapping:
                        bn_d = cfg._english2bangla2_digits_mapping[en_d]
                        d_character.append(bn_d)
                    else:
                        d_character.append(en_d)
                bn_digits = "".join(d_character)
                bn_data_list.append(bn_digits)
            else:
                bn_data_list.append(d_l)
        # print("bn_data_list : ", bn_data_list)
        return bn_data_list

    def month_spliting_issue_solver(self, org_text, reference_data):
        date = reference_data
        text = org_text
        # Find all occurrences of the date in the text
        matches = [match.start() for match in re.finditer(date, text)]
        extracted_text_list = []
        for m in matches:
            start = m
            end = m + len(date)
            # Extend the end position until the next space or end of the text
            while end < len(text) and text[end] != " ":
                end += 1
            # Print the start and end positions and the extracted text
            # print(f"Start: {start}, End: {end}")
            # print(f"Extracted Text: '{text[start:end]}'")
            # extracted_text_list.append(text[start:end])
            return text[start:end]
        # Print the positions of the date occurrences
        return None

    def validate_may_connected_with_charater_and_is_year(
        self, org_text, reference_data
    ):

        date = reference_data
        text = org_text
        # Find all occurrences of the date in the text
        matches = [match.start() for match in re.finditer(date, text)]
        status = True
        for m in matches:
            start = m
            end = m + len(date)
            while start > 0 and text[start - 1] != " ":
                start -= 1
            matches = re.findall(self.year_pattern, text[start:end])

            if " " in text[start:end]:
                chunk_d = text[start:end].split()
                for c_d in chunk_d:
                    if (
                        c_d in cfg.data["bn"]["months"]
                        or c_d in cfg.data["bn"]["option_name"]
                    ):
                        return True
                    else:
                        return False
            else:
                return True

    def check_date_format(self, text):
        pattern1 = (
            r"^\d{4}\s*-\s*\d{1,2}\s*-\s*\d{1,2}$"  # e.g., 2023 - 04 - 05 or 2023-4-5
        )
        pattern2 = (
            r"^\d{1,2}\s*-\s*\d{1,2}\s*-\s*\d{4}$"  # e.g., 06 - 04 - 2023 or 6-4-2023
        )
        pattern3 = r"^\d{1,2}/\d{1,2}/\d{4}$"  # e.g., 04/01/2023 or 4/1/2023
        pattern4 = (
            r"^\d{4}\s*/\s*\d{1,2}\s*/\s*\d{1,2}$"  # e.g., 2023 / 04 / 01 or 2023/4/1
        )

        # Check if the text matches any of the patterns
        if (
            re.match(pattern1, text)
            or re.match(pattern2, text)
            or re.match(pattern3, text)
            or re.match(pattern4, text)
        ):
            return True
        return False

    def extract_between_dashes(text):
        # Define the regex pattern
        # pattern = r'^.{4}-(.{2})$'
        pattern = r"^\d{4}\s*-\s*\d{2}$"
        match = re.match(pattern, text)

        print(match[0])

    def replace_date_processing(self, text):
        text = self.extract_year(text)
        # print(text)
        original_text = text
        r_text = text
        # print("original_text : ", original_text)
        dates = dt.get_dates(text)
        for date in dates:
            # print("date1 : ", date)
            r_date = self.month_spliting_issue_solver(original_text, date)
            # print("r_date ", r_date)
            n_status = self.validate_may_connected_with_charater_and_is_year(
                original_text, r_date
            )
            # print(n_status, r_date)
            if n_status == False:
                continue
            if r_date is None:
                r_data = date
            # r_date = date
            date_list = self.check_date_format_exception_case(date.strip())
            # print("date_list : ", date_list)
            if date_list:
                formated_date = self.dp.date_processing(date_list)

                bn_data_list = self.english_date_to_bangla_date(date_list)
                for k, v in formated_date.items():
                    if v in bn_data_list:
                        key = k if "txt" in k else f"txt_{k}"
                        index = bn_data_list.index(v)
                        bn_data_list[index] = formated_date[key]
                process_date = " ".join(bn_data_list).strip()
                original_text = original_text.replace(r_date, " " + process_date + " ")
            else:
                date = self.add_spaces_to_numbers(date)

                status = True
                if " " in date:

                    status, text = self.date_formate_validation(date, text)

                    status = self.check_date_format(date)

                # print(f"status {status} date : {date}")
                if status:
                    formated_date = self.dp.date_processing(date)
                    # print("formated_date : ", formated_date)
                    original_date = date
                    date_list = [i for i in date.split(" ") if i.strip()]
                    bn_data_list = self.english_date_to_bangla_date(date_list)
                    for k, v in formated_date.items():
                        if v in bn_data_list:
                            key = k if "txt" in k else f"txt_{k}"
                            index = bn_data_list.index(v)
                            bn_data_list[index] = formated_date[key]

                    process_date = " ".join(bn_data_list).strip()
                    if process_date.isdigit():
                        continue
                    else:
                        original_text = original_text.replace(
                            r_date, " " + process_date + " "
                        )
        _only_years = re.findall(self.year_pattern, original_text)
        for y in _only_years:
            if y.isdigit():
                continue
            else:
                original_text = original_text.replace(
                    y, " " + self.npr.year_in_number(y) + " "
                )

        f_index = 0
        for full_name in cfg.data["en"]["months"]:
            if full_name in original_text or full_name.capitalize() in original_text:
                bn_name = cfg.data["bn"]["months"][f_index]
                original_text = original_text.replace(full_name, bn_name)
                original_text = original_text.replace(full_name.capitalize(), bn_name)
            f_index += 1
        s_index = 0
        for short_name in cfg.data["en"]["option_name"]:
            # print(short_name)
            if short_name in original_text or short_name.capitalize() in original_text:
                bn_name = cfg.data["bn"]["months"][s_index]
                original_text = original_text.replace(short_name, bn_name)
                original_text = original_text.replace(short_name.capitalize(), bn_name)

            s_index += 1
        return original_text

    def processing(self, text):
        text = self.exception_year_processing(text)
        text = pne.phn_num_extractor(text)
        text = self.unwanted_puntuation_removing(text)
        text = self.collapse_whitespace(text)
        text = self.year_formation(text)
        text = self.expand_symbols(text)
        text = self.expand_abbreviations(text)
        text = self.expand_position(text)
        text = self.extract_currency_amounts(text)
        text = self.replace_date_processing(text)
        text = self.npr.number_processing(text)
        text = self.collapse_whitespace(text)
        return text

    def data_normailization(self, text):

        text = self.exception_year_processing(text)
        text = self.unwanted_puntuation_removing(text)
        text = self.collapse_whitespace(text)
        text = self.year_formation(text)
        text = self.replace_date_processing(text)
        text = self.collapse_whitespace(text)
        return text


class EmojiRemoval:

    def __init__(self):
        self.regex_to_remove_emoji = re.compile(
            "["
            "\U0001F600-\U0001F64F"  # emoticons
            "\U0001F300-\U0001F5FF"  # symbols & pictographs
            "\U0001F680-\U0001F6FF"  # transport & map symbols
            "\U0001F1E0-\U0001F1FF"  # flags (iOS)
            "\U00002500-\U00002BEF"  # chinese char
            "\U00002702-\U000027B0"
            "\U00002702-\U000027B0"
            "\U000024C2-\U0001F251"
            "\U0001f926-\U0001f937"
            "\U00010000-\U0010ffff"
            "\u2640-\u2642"
            "\u2600-\u2B55"
            "\u200d"
            "\u23cf"
            "\u23e9"
            "\u231a"
            "\ufe0f"  # dingbats
            "\u3030"
            "]+",
            re.UNICODE,
        )
        self.tp = TextParser()

    def remove_emoji(self, text):
        text = text.replace(" , ", ", ")
        text = re.sub(self.regex_to_remove_emoji, " ", text)
        text = self.tp.collapse_whitespace(text)
        return text


# class EmojiReplacer:
#     def __init__(self):
#         self.translator = Translator()

#     def replace_emoji(self, text):
#         txt = self.translator.translate(text).text
#         # print(txt)
#         return text if len(txt)==0 else txt

if __name__ == "__main__":

    text = "রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম, ১২৩৪ শতাব্দীতে ¥২০৩০.১২৩৪ বিবিধ  বাকেরগঞ্জ উপজেলার প্রায় 40 ভাগের পেশাই চাষাবাদ 80 and 40 ২২"
    tp = TextParser()
    text = tp.processing(text)
    print(text)



import re
from .config import Config as cfg


class PhoneNumberExtractor:

    def __init__(self):
        self.number_extention = ["+88", "+৮৮"]
        self.phn_number = [
            "017",
            "015",
            "013",
            "016",
            "018",
            "019",
            "০১৭",
            "০১৫",
            "০১৩",
            "০১৬",
            "০১৮",
            "০১৯",
        ]
        # self.pattern = r'(?:\+?৮৮)?০১[৩-৯][০-৯]{2}-?[০-৯]{6}|(?:\+88)?01[3-9]\d{2}-?\d{6}'
        self.pattern = r"(?:\+?৮৮)?০১[৩-৯][০-৯]{2}-?[০-৯]{6}(?=[, ]|$)|(?:\+88)?01[3-9]\d{2}-?\d{6}(?=[, ]|$)"
        self.plux = {"+": "প্লাস"}

    # def checking_prefix()

    def contains_only_english(self, input_string):
        # Check if all characters in the string are English (ASCII) characters
        return all(ord(char) < 128 for char in input_string)

    def get_number2word(self, num):
        num_mapping = {
            **cfg.data["en"]["number_mapping"],
            **cfg.data["bn"]["number_mapping"],
        }
        if num in num_mapping:

            return num_mapping[num]
        return num

    def label_repeats(self, number):
        result = []
        digit_map = cfg.data["en"]["number_mapping"]
        c_number = []
        for i in number:
            status = self.contains_only_english(i)
            if status == False:
                if i in cfg._bangla2english_digits_mapping:
                    c_n = cfg._bangla2english_digits_mapping[i]
                    c_number.append(c_n)
                else:
                    c_number.append(i)
            else:
                c_number.append(i)
        number = c_number
        i = 0
        n = len(number)
        while i < n:
            if i + 2 < n and number[i] == number[i + 1] == number[i + 2]:
                result.append(cfg.special_map[number[i] * 3])
                i += 3
            elif i + 1 < n and number[i] == number[i + 1]:
                result.append(cfg.special_map[number[i] * 2])
                i += 2
            else:
                if number[i] in digit_map:
                    # print("number[i] : ", number[i], type(number[i]))
                    result.append(digit_map[number[i]])
                elif number[i] == "-":
                    result.append(" ")
                i += 1

        return " ".join(result)

    def phn_num_extractor(self, text):
        phone_numbers = re.findall(self.pattern, text)

        sorted_matches = sorted(phone_numbers, key=len, reverse=True)
        for phone_number in sorted_matches:

            modify_phone_number = "".join(re.split(r"[- ]", phone_number))

            # print(modify_phone_number)

            first_3_character = modify_phone_number.strip()[:3]
            if (
                first_3_character in self.number_extention
                or first_3_character in self.phn_number
                or 11 <= len(modify_phone_number)
                or len(modify_phone_number) <= 14
            ):

                temp_string = ""
                if "+" == modify_phone_number[0]:
                    # print(modify_phone_number[1:])
                    temp_string = (
                        "প্লাস" + " " + self.label_repeats(modify_phone_number[1:])
                    )
                else:
                    repate_string = self.label_repeats(modify_phone_number)
                    # print(repate_number)
                    temp_string = " " + repate_string

            phone_number_string = temp_string.strip()

            text = text.replace(phone_number, phone_number_string)
            # print(phone_number_string)
        # print(text)
        return text


if __name__ == "__main__":

    text = "ডিজিটাল রেজিস্ট্রেশন সার্টিফিকেট সংক্রান্ত 01790540211124562 যোগাযোগ করতে হলে 01790-540211 অথবা 01790-541111 নম্বরে যোগাযোগ করতে হবে 01790540211, +8801790540211, ০১৭৯০৫৪০২১১, +৮৮০১৭৯০৫৪০২১১"
    pne = PhoneNumberExtractor()
    process_text = pne.phn_num_extractor(text)
    print("input : ", text)
    print("output : ", process_text)
